// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

// Include the std::vector:
#include <vector>

// Include our Mat4f class:
#include "src/math/Mat4f.h"

// Include our Mesh class:
#include "src/primitives/Triangle.h"

// Include our Mesh class:
#include "src/Mesh.h"

// Include our Shader class:
#include "src/Shader.h"

// Include our world class:
#include "src/game/World.h"

/**
 * This is a spinning top class which represents a renderable
 * object in the world.
 */
class SpinningTop {
private:
    // The mesh of this spinning top.
    Mesh mesh;

    // The shader for the mesh of this spinning top:
    Shader shader;

    // The color in which the sphere should be rendered
    // (this should be the color of the ground).
    Vec4f color;

    // The model matrix of this spinning top which
    // describes where this spinning top is in the world
    // (and where it's mesh should be rendered).
    Mat4f modelMatrix;

    // The current position of the spinning top:
    Vec4f position;

    // The moving direction (considering only XZ):
    Vec4f movingDirection;

    // Smoothed moving direction (for a better camera view):
    Vec4f smoothedDirection;

    // The speed with which the spinning top moves in the
    // movingDirection:
    float speed = 0.008f;

public:
    /**
     * Creates a spinning top which contains the given mesh.
     */
    SpinningTop(Mesh mesh, Shader shader, Vec4f initialPosition);

    /**
     * This function is called to calculate the new state
     * of the pawn, e.g. the model matrix.
     */
    void simulate(World& world);

    /**
     * This function renders the game object with the given
     * view and projection matrix.
     */
    void render(Mat4f view, Mat4f projection);

    /**
     * Returns the model matrix.
     */
    Mat4f getModelMatrix();
};
