// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

// Include the std::vector:
#include <vector>

// Include our Mat4f class:
#include "src/math/Mat4f.h"

// Include our Mesh class:
#include "src/primitives/Triangle.h"

// Include our Mesh class:
#include "src/Mesh.h"

// Include our Shader class:
#include "src/Shader.h"

// Forward declaration of class SpinningTop (this is a C++ specific
// thing which is required when two classes use each other. This is
// not required in Java, etc.):
class SpinningTop;

/**
 * A world class.
 */

struct World {
    // A 'list' which contains all the spinning tops in the world:
    std::vector<SpinningTop> spinningTops;

    // Ground mesh of the world:
    Mesh groundMesh;

    // A shader which is used to render the ground:
    Shader groundShader;

    /**
     * Constructs the world with a mesh and shader for the ground.
     */
    World(Mesh groundMesh, Shader groundShader);

    /**
     * Renders the groundMesh and calls the simulate and render
     * method on all actors.
     */
    void render(Mat4f view, Mat4f projection);
};
